/* Copyright (C) NHR@FAU, University Erlangen-Nuremberg.
 * All rights reserved.
 * Use of this source code is governed by a MIT-style
 * license that can be found in the LICENSE file.*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "comm.h"
#include "progress.h"
#include "solver.h"
#include "timing.h"

static FILE* initResidualWriter()
{
    FILE* fp;
    fp = fopen("residual.dat", "w");

    if (fp == NULL) {
        printf("Error!\n");
        exit(EXIT_FAILURE);
    }

    return fp;
}

static void writeResidual(FILE* fp, double ts, double res)
{
    fprintf(fp, "%f, %f\n", ts, res);
}

int main(int argc, char** argv)
{
    int rank;
    double timeStart, timeStop;
    Parameter p;
    Solver s;

    commInit(s.comm, argc, argv);
    initParameter(&p);

    FILE* fp;
    if (commIsMaster(s.comm)) fp = initResidualWriter();

    if (argc != 2) {
        printf("Usage: %s <configFile>\n", argv[0]);
        exit(EXIT_SUCCESS);
    }

    readParameter(&p, argv[1]);
    commPartition(s.comm, p.jmax, p.imax);
    if (commIsMaster(s.comm)) {
        printParameter(&p);
    }

    initSolver(&s, &p);
    // #ifndef VERBOSE
    //     initProgress(s.te);
    // #endif

    double te  = p.te;
    double t   = 0.0;
    double res = 0.0;

    timeStart = getTimeStamp();
    while (t <= te) {

        if (commIsMaster(s.comm)) writeResidual(fp, t, res);

        t += p.dt;

#ifdef VERBOSE
        if (commIsMaster(s.comm)) {
            printf("TIME %f , TIMESTEP %f\n", t, d.dt);
        }
#else
        printProgress(t);
#endif
    }
    timeStop = getTimeStamp();
#ifndef VERBOSE
    stopProgress();
#endif
    if (commIsMaster(s.comm)) {
        printf("Solution took %.2fs\n", timeStop - timeStart);
    }
    if (commIsMaster(s.comm)) fclose(fp);
    commFinalize(s.comm);
    return EXIT_SUCCESS;
}
